// -*- Java -*- Last
/*!
 * @file  SafeModuleImpl.java
 * @brief AnzenJohoTeikyoModule
 * @date  $Date$
 * $Id$
 */
import java.io.*;
import java.util.Calendar;
import java.util.Locale;

import RTC.TimedLong;
import jp.go.aist.rtm.RTC.DataFlowComponentBase;
import jp.go.aist.rtm.RTC.Manager;
import jp.go.aist.rtm.RTC.port.OutPort;
import jp.go.aist.rtm.RTC.util.DataRef;
import jp.go.aist.rtm.RTC.port.CorbaConsumer;
import jp.go.aist.rtm.RTC.port.CorbaPort;
import org.omg.PortableServer.POAPackage.ObjectNotActive;
import org.omg.PortableServer.POAPackage.ServantAlreadyActive;
import org.omg.PortableServer.POAPackage.WrongPolicy;
import RTC.ReturnCode_t;
import jp.co.ritecs.DataInterpretationModule.RTC.diManagement;
import jp.co.ritecs.DataInterpretationModule.RTC.diTag;
import jp.co.ritecs.DataInterpretationModule.RTC.diObject;
import jp.co.ritecs.DataInterpretationModule.RTC.diRobot;

/*!
 * @class SafeModuleImpl
 * @brief AnzenJohoTeikyoModule
 *
 */
import SIP2.*;
import static java.lang.Math.*;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;
import java.util.Calendar;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import jp.co.ritecs.DataInterpretationModule.RTC.*;
import jp.co.ritecs.DataInterpretationModule.RTC.diData.*;

public class SafeModuleImpl extends DataFlowComponentBase {

  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
	public SafeModuleImpl(Manager manager) {  
        super(manager);
        // <rtc-template block="initializer">
        m_upperlimit_val = new TimedLong(new RTC.Time(0,0),0);			//C
        m_upperlimit = new DataRef<TimedLong>(m_upperlimit_val);
        m_upperlimitOut = new OutPort<TimedLong>("upperlimit", m_upperlimit);
        m_signal_val = new TimedLong(new RTC.Time(0,0),0);				//C
        m_signal = new DataRef<TimedLong>(m_signal_val);
        m_signalOut = new OutPort<TimedLong>("signal", m_signal);
        m_spRobotPathPort = new CorbaPort("spRobotPath");
        m_diManagementPort = new CorbaPort("diManagement");
        m_diTagPort = new CorbaPort("diTag");
        m_diObjectPort = new CorbaPort("diObject");
        m_diRobotPort = new CorbaPort("diRobot");
        // </rtc-template>
    }

    /**
     *
     * The initialize action (on CREATED->ALIVE transition)
     * formaer rtc_init_entry() 
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onInitialize() {
        // Registration: InPort/OutPort/Service
        // <rtc-template block="registration">
        
        // Set OutPort buffer
        addOutPort("upperlimit", m_upperlimitOut);
        addOutPort("signal", m_signalOut);
        
        // Set service provider to Ports
        try {
        	m_spRobotPathPort.registerProvider("spRobotPath", "SIP2::spRobotPath", m_spRobotPath);
        } catch (ServantAlreadyActive e) {
            e.printStackTrace();
        } catch (WrongPolicy e) {
            e.printStackTrace();
        } catch (ObjectNotActive e) {
            e.printStackTrace();
        }
        
        // Set service consumers to Ports
        m_diManagementPort.registerConsumer("if_diManagement", "jp::co::ritecs::DataInterpretationModule::RTC::diManagement", m_if_diManagementBase);
        m_diTagPort.registerConsumer("if_diTag", "jp::co::ritecs::DataInterpretationModule::RTC::diTag", m_if_diTagBase);
        m_diObjectPort.registerConsumer("if_diObject", "jp::co::ritecs::DataInterpretationModule::RTC::diObject", m_if_diObjectBase);
        m_diRobotPort.registerConsumer("if_diRobot", "jp::co::ritecs::DataInterpretationModule::RTC::diRobot", m_if_diRobotBase);
        
        // Set CORBA Service Ports
        addPort(m_spRobotPathPort);
        addPort(m_diManagementPort);
        addPort(m_diTagPort);
        addPort(m_diObjectPort);
        addPort(m_diRobotPort);
        // </rtc-template>
        return super.onInitialize();
    }

    /***
     *
     * The finalize action (on ALIVE->END transition)
     * formaer rtc_exiting_entry()
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onFinalize() {
        return super.onFinalize();
    }

    /***
     *
     * The startup action when ExecutionContext startup
     * former rtc_starting_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onStartup(int ec_id) {
		Open();
        return super.onStartup(ec_id);
    }

    /***
     *
     * The shutdown action when ExecutionContext stop
     * former rtc_stopping_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onShutdown(int ec_id) {
//        return super.onShutdown(ec_id);
//    }

    /***
     *
     * The activated action (Active state entry action)
     * former rtc_active_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onActivated(int ec_id) {
        return super.onActivated(ec_id);
    }

    /***
     *
     * The deactivated action (Active state exit action)
     * former rtc_active_exit()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onDeactivated(int ec_id) {
//        return super.onDeactivated(ec_id);
//    }

    /***
     *
     * The execution action that is invoked periodically
     * former rtc_active_do()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    int ji = 0;
    @Override
    protected ReturnCode_t onExecute(int ec_id) {
    	// System.out.println("onExecute.");
        m_if_diManagement = m_if_diManagementBase._ptr();
        m_if_diTag = m_if_diTagBase._ptr();
        m_if_diObject = m_if_diObjectBase._ptr();
        m_if_diRobot = m_if_diRobotBase._ptr();
        
        double bluespeed = 1000;					//Mx(1000m/s)
        double yellowspeed = 400;					//Mx(400m/s)
        double redspeed = 0;					//ԐMx(0m/s)
        
    	
    	//܂AZbgĂg|WJ}bvA{bgpX𓾂
    	//ꉞ擾Ă邪AZbgꂽ̂ύXȂȂxł悢
    	//ݒ肳ĂmapApathNodeslengthǂݍ
    	int[] lengthbox = new int[2];			//lengthbox0maplength,lengthbox1pathlength
    	boolean po = m_spRobotPath.getlength(lengthbox);
    	
    	if(lengthbox[0]==0 || lengthbox[1] ==0){				//܂}bvĂȂƂ
    		System.out.print("g|WJ}bvA{bgoHZbgĂ܂.Abp[~bg10𔭐MD"+"\n");
        	m_upperlimit_val.data = 10;
        	m_signal_val.data = 10;
        	m_upperlimitOut.write();
        	m_signalOut.write();
            return super.onExecute(ec_id);
    	}
//   	if (po == true){										//}bvm
//       		// System.out.print("Zbgꂽmap̃TCY" + lengthbox[0] + "Apath̃TCY" + lengthbox[1] +"ł\n");
//    	}

    	Node[] tizu = new Node[lengthbox[0]];			//ݒ肳Ămap,pathNodesƓTCYNode
    	Node[] root = new Node[lengthbox[1]];			//gtizu  root
    	
    	boolean mapget = mapgetting(tizu, root);							//map,pathNodesǂݍ
    	if(mapget == false){
    		System.out.print("g|WJ}bv̓ǂݍ݂Ɏs܂.Abp[~bg10𔭐MD"+"\n");
        	m_upperlimit_val.data = 10;
        	m_signal_val.data = 10;
        	m_upperlimitOut.write();
        	m_signalOut.write();
            return super.onExecute(ec_id);
    	}
   		
    	//Currentstatsǂݍ
   		CurrentStatus sute = new CurrentStatus();
    	po = m_spRobotPath.getCurrentStatus(sute);
    	if (po == true){
      		System.out.print("Current NodeID:"+ sute.NodeID + " status:" + sute.status +"\n");
    	}    	
    	int goalnum = targetselect(tizu, root, sute.NodeID);				//tizu[goalnum]ړINode
    	if(goalnum == 999999){
        	System.out.print("ŏIm[hɓBĂ܂BAbp[~bg10𔭐MD\n");
        	m_upperlimit_val.data = 10;
        	m_signal_val.data = 10;
        	m_upperlimitOut.write();
        	m_signalOut.write();
        	Date myDate=new Date();System.out.println("SafetyModeule:OK timestamp:"+myDate.getTime()); return super.onExecute(ec_id);
    	}
    	System.out.print("ݎ̖ړInm[h" + tizu[goalnum].id +"\n"); 

    	
    	//ŃT[rX|[gA{bgx,yƐlx,y,v擾
    	double[] rob = new double[2];				//0:x,1:yCPʂmm
    	double[][] hum = new double[5][4];		//[l][0:x,1:y,2:vx,3:vy]CPʂmmň.Ƃ肠max5lB
    	
    	int grhi =  GetRobotHumanInformation(rob, hum);
    	if(grhi==-1){
    		System.out.println("{bg̃f[^ǂݍ߂܂łD\͂AAbp[~bg0𔭐MD");
        	m_upperlimit_val.data = 0;
        	m_signal_val.data = 0;
        	m_upperlimitOut.write();
        	m_signalOut.write();
        	Date myDate=new Date();System.out.println("SafetyModeule:OK timestamp:"+myDate.getTime()); return super.onExecute(ec_id);
    	}else if(grhi ==0){
			System.out.println("humanoł܂łBlԂ͂ȂƔfAbp[~bg10𔭐MD");
        	m_upperlimit_val.data = 10;
        	m_signal_val.data = 10;
        	m_upperlimitOut.write();
        	m_signalOut.write();
        	Date myDate=new Date();System.out.println("SafetyModeule:OK timestamp:"+myDate.getTime()); return super.onExecute(ec_id);
    	}else if(grhi == 5){
    		System.out.println("l܂BAbp[~bg0𔭐MD");
        	m_upperlimit_val.data = 0;
        	m_signal_val.data = 0;
        	m_upperlimitOut.write();
        	m_signalOut.write();
        	Date myDate=new Date();System.out.println("SafetyModeule:OK timestamp:"+myDate.getTime()); return super.onExecute(ec_id);
    	}else{
    		System.out.println(grhi + "lhumano܂");
    	}

    	
    	//Փ˔
    	//:\ԊuisɊ肩j,robspeedi{bgXs[hmm/sjCrobdata[2],humdata[4][4]i{bglԍWSĂƂ͌Ȃ), goalnodeiړInm[hj,humanl
    	//Ԓl:trueƏՓ. falseƏՓ˂
    	
    	boolean langis = collisioncheck(1.0, bluespeed, rob, hum, tizu[goalnum],grhi);   //qxŐi񂾏ꍇ̊`FbN
    	if(langis==false){								//falseAĂꍇAՓ˂͗\Ȃ
    		m_upperlimit_val.data = 10;
        	m_signal_val.data = 10;
    		System.out.println("Abp[~bg10𔭐MD");
    	}else{											//trueł͊\Ă
    		langis = collisioncheck(1.0, yellowspeed, rob, hum, tizu[goalnum],grhi);			//sĐi񂾏ꍇ̃`FbN
        	if(langis==false){								//sxŏՓ˂\Ȃ
        		m_upperlimit_val.data = 4;
        		m_signal_val.data = 4;
        		System.out.println("Abp[~bg4𔭐MD");
        	}else{											//sxłՓ˂\ꂽ
        		m_upperlimit_val.data = 0;
        		m_signal_val.data = 0;
        		System.out.println("Abp[~bg0𔭐MD");
        	}
 		}
    	m_upperlimitOut.write();
    	m_signalOut.write();

    	
    	Date myDate=new Date();System.out.println("SafetyModeule:OK timestamp:"+myDate.getTime());
    	return super.onExecute(ec_id);
    }

    
    private boolean collisioncheck(double predictrate, double robspeed, double[] robdata, double[][] humdata, Node goalnode, int numofhuman){
    	//Փ˔
    	//:\ԊuisɊ肩j,robsppedi{bgXs[hmm/sjCrobdata[2],humdata[4][4]i{bglԍWBSƂ͌ȂjCgoalnodeiړInm[hjAnumofhuman(l
    	//YŏՓ˂ȂtrueԂBՓ˂ȂȂfalse	Ԃ.
 		
		int syoutotu = 0;									//Փ˂0or1BՓ˂\1ɑAtrue𓱂
		//lhumanf[^ŉ񂷁Bsyoutotu1ɂȂ߂邱ƂŌvZʌ炷
    	for(int number=0; (number<numofhuman)&&syoutotu==0 ;number++){   
    		//humdataɁAp[eBNz𐶐ApredictrateɈړė\
        	int PN =100;										//p[eBN
    		double[][] particle = new double[4][PN];			//100̃p[eBNB[0][1]x,yW [2][3]x,y,v
    		//boxmullerɂĐKz
    		double sigma = 30.0;		//W΍
    		double r1,r2,z1,z2;
    	
    		for(int i=0; i<PN ; i++){
    			do{
    				r1=random();
    				r2=random();
    			}while(r1<=0.0 || r2<=0.0);
    			z1=sigma*sqrt(-2*log(r1))*cos(2*PI*r2)+ humdata[number][0];
    			z2=sigma*sqrt(-2*log(r1))*sin(2*PI*r2)+ humdata[number][1];
    			particle[0][i] = z1;
    			particle[1][i] = z2;
    			particle[2][i] = humdata[number][2];
    			particle[3][i] = humdata[number][3];
    		}								//particle[][]ɁAhumdata[x,y]𒆐SɏKzAxNg(x)}
    									//\Nhumparticle
    		//\񐔂߂
    		double predicttime;				//\ԁBmax 5[s]
    		double timetogoal =sqrt( (goalnode.xPos - robdata[0] )*( goalnode.xPos - robdata[0] ) + ( goalnode.yPos - robdata[1] )*( goalnode.yPos - robdata[1] ) ) / robspeed;	//ݒ葬xŁA݃{ʒuS[m[h܂ł鎞
   			if(timetogoal > 5.0){
   				predicttime = 5.0;
   			}else if(timetogoal == 0){
   				return false;
    		}else{
   				predicttime = timetogoal;
   			}
   			int predictkaisu = (int)( predicttime / predictrate);			//\tF[Y񂷉
   	    	// System.out.println("̃m[h܂"+timetogoal+"s邽߁C"+predicttime+"s̗\܂"+predictkaisu +"̗\tF[Y񂵂܂");
   	    	
   			double r,sita,ransita;				//Ɨpϐ
   			boolean atari;				//Ɨpϐ
   			double sigma2 = 0.039;			//W΍
   			for(int p=1; p<=predictkaisu; p++){
   				//pɉA{bg̗\ʒuvZiS[܂œ^j
   				double robotpredictx = (( goalnode.xPos - robdata[0] ) / timetogoal) * (predictrate*p)+robdata[0];
   				double robotpredicty = (( goalnode.yPos - robdata[1] ) / timetogoal) * (predictrate*p)+robdata[1];
   	//			// System.out.println("{gx"+robotpredictx+"{gy"+robotpredicty);
   				//pɉlԂ̗\p[eBNQvZArobotpredictionx,yƕӂ蔻
   				for(int i=0; i<PN ; i++){	
   					r = (particle[2][i]*particle[2][i]) + (particle[3][i]*particle[3][i]);			/*mZo*/
   					r = sqrt(r);			//rɁA܂ł̃xNg̃m
   					if(r==0){
   						sita =0;
   					}else{
   						sita = asin(particle[3][i]/r);				//sitaɁA܂ł̃xNg̊px
   					}
   					
   					if(particle[2][i] < 0){						//x̏ꍇPIāA360xΉ
   						sita = PI - sita;
   					}
  
   					do{										//܂ł̕烉_Ȋpx𐶐
   						z1=random();
   						z2=random();
   					}while(z1<=0.0 || z2<=0.0);
   					ransita =  sigma2*sqrt(-2.0*log(z1))*cos(2.0*PI*z2) + sita;	//KŃ_Ȋpx𐶐
   					particle[2][i] = r*cos(ransita);			//ꂽ_xNgx,yW
   					particle[3][i] = r*sin(ransita);
   					particle[0][i] = particle[0][i] + particle[2][i];			//ꂽxNgɁAp[eBis
   					particle[1][i] = particle[1][i] + particle[3][i];
        		
   					atari = atarihantei(robotpredictx, robotpredicty, particle[0][i],particle[1][i]);	//pڂ̃tF[Yɂ郍{bgAl蔻
   					if(atari==true){
   						syoutotu =1;
//   						// System.out.println(robotpredictx+","+robotpredicty+","+particle[0][i]+","+particle[1][i]);
   					}
   				}	
   			}
//   			meyasu(predictkaisu, PN, particle);				//mF
    	}

    	
    	//max5bԁApredicrateɏ]APN̃p[eBN𐶐ė\𑱂Asyoutotu1ɂȂ邱Ƃ΁A͂ǂŊ\ꂽƂ
    	//0̂܂܂ȂAxatarihanteitruełȂA܂花\Ȃƌ
    	if(syoutotu==1){
    		return true;
    	}else{
    		return false;
    	}
    	
    }

    private void meyasu(int predictkaisu, int PN, double[][] particle){
    	//particle̒S߂B̖̂ڈʒuԂmFp
    	double xxx=0,yyy=0;
    	for(int i=0;i<PN;i++){
    		xxx = particle[0][i] + xxx;
    		yyy = particle[1][i] + yyy;
    	}
    	xxx = xxx/PN;
    	yyy = yyy/PN;
    	// System.out.println("\p[eBNQςAW  "+xxx+","+yyy);
    }
    
    
    private boolean atarihantei(double robx,double roby, double humx, double humy){
    	//Փ˕ӂ蔻胁\bh
    	//roba300mm,huma250mmƒ`Arob~hum~Ă邩𔻒
    	double robot_r =300.0;
    	double human_r = 250.0;
    	double p = ((robx - humx)*(robx - humx)) + ((roby - humy)*(roby - humy));
    	double q = (robot_r + human_r)*(robot_r + human_r);
    	if(p <= q){
        	return true;	
    	}else{
    		return false;
    	}
    }
    
    private int targetselect(Node[] mapdata, Node[] pathroot, long nowpoint){
    	//nowpointƓpathroot.idTA̎pathroot.idppɕ荞
    	//pathroot.idmapdata̒mapdata[?]?Ԃ
    	int pp=0,qq=0;

    	if(nowpoint==pathroot[pathroot.length-1].id){		//S[
    		return 999999;		//~R[h
    	}
    	
    	for (int i = 0; i < pathroot.length; i++) {	
    		if(pathroot[i].id == nowpoint){
    			pp = i+1;
    		}
    	}
    	pp = pathroot[pp].id;						//̎_łppɂ́A̔ԍĂ
    	for (int i = 0; i < mapdata.length; i++) {	
    		if(mapdata[i].id == pp){
    			qq = i;
    		}
    	}
       	return qq;					
    }
    
    private boolean mapgetting(Node[] mapcase, Node[] pathcase){
  		mapcase[0] = new Node();
    	mapcase[0].name = "question";					//NGX`R[h

    	boolean po = m_spRobotPath.setRobotPath(mapcase,pathcase);
    	if (po == true){
 //   		// System.out.print("}bvȂǂǂݍ݂܂\n");
    		return true;
    	}else{
    		return false;
    	}
    }
    
    
	//ȉA񋟃W[ڑ\bh
    
	private void Open()
	{
		// System.out.println("open.");
		int res = m_if_diManagement.Open();
		if (res == 0) {
			// System.out.println("ok.");
		} else {
			// System.out.println("error[" + res + "]");
		}
	}

	private int GetRobotHumanInformation(double[]rob, double[][] hum){
		//getvisionInfoID烍{bg̃f[^擾
		//tagquestiońAGAɂQ̃f[^̂ŁA̒烍{bg̃f[^҂s҂̃f[^
	
		
		//肩烍{bglԂ
		String objectid;
		objectid = "1";				//getviosionInfoID1ɏɃ{bgf[^
		
		diVisionInfoHolder vision = new diVisionInfoHolder();
		vision.value = new diVisionInfo();
		long res = m_if_diObject.getObject(objectid, vision);
		if (res == 0) {
			System.out.println("Robot̎Ȉʒuf[^擾ok.");
		} else {
			System.out.println("error[" + res + "]@{bgȂ");
			return -1;		//{bgȂ
		}	
		
		diVisionInfo info = new diVisionInfo();
		info = vision.value;
		rob[0] = info.coordinateX;
		rob[1] = info.coordinateY;
		// System.out.println("{bgf[^ǂݍ݁FW("+rob[0] +","+ rob[1] + ")");
		int ij=0;								//lϐ
    	double[] zahyou = new double[4];		//WƔz
    	double[][] humrob = new double[5][4];	//lԂƃ{bg𕪂Ô
    	boolean tagques;
		tagques = tagquestion("person1", zahyou);
		if(tagques == true){					//id16 (human1)̂œ
				humrob[ij][0] = zahyou[0];
				humrob[ij][1] = zahyou[1];
				humrob[ij][2] = zahyou[2];
				humrob[ij][3] = zahyou[3];
				ij = ij +1;							//ꂽ̂Ői߂
				System.out.println("Person1 Yes.");
		}else{
				System.out.println("Person1 No.");
		}
		tagques = tagquestion("person2", zahyou);
		if(tagques == true){					//human2̂œ
				humrob[ij][0] = zahyou[0];
				humrob[ij][1] = zahyou[1];
				humrob[ij][2] = zahyou[2];
				humrob[ij][3] = zahyou[3];
				ij = ij +1;							//ꂽ̂Ői߂
				System.out.println("Person2 Yes.");
		}else{
				System.out.println("Person2 No.");
		}
		tagques = tagquestion("person3", zahyou);
		if(tagques == true){					//id2(human3)̂œ
				humrob[ij][0] = zahyou[0];
				humrob[ij][1] = zahyou[1];
				humrob[ij][2] = zahyou[2];
				humrob[ij][3] = zahyou[3];
				ij = ij +1;							//ꂽ̂Ői߂
				System.out.println("Person3 Yes.");
		}else{
				System.out.println("Person3 No.");
		}
		tagques = tagquestion("person4", zahyou);
		if(tagques == true){					//id23(human4)̂œ
				humrob[ij][0] = zahyou[0];
				humrob[ij][1] = zahyou[1];
				humrob[ij][2] = zahyou[2];
				humrob[ij][3] = zahyou[3];
				ij = ij +1;							//ꂽ̂Ői߂
				System.out.println("Person4 Yes.");
		}else{
				System.out.println("Person4 No.");
		}
		tagques = tagquestion("person5", zahyou);
		if(tagques == true){					//id3(human5)̂œ
				humrob[ij][0] = zahyou[0];
				humrob[ij][1] = zahyou[1];
				humrob[ij][2] = zahyou[2];
				humrob[ij][3] = zahyou[3];
				ij = ij +1;							//ꂽ̂Ői߂
				System.out.println("Person5 Yes.");
		}else{
				System.out.println("Person5 No.");
		}
		
				//ijɂhumrob̓e̐(0`5)
				//humrobɂ͍ő5̍WAlԂƃ{bg
		// System.out.println("ij = "+ij);
		// System.out.println(humrob[0][0] + "," +humrob[0][1] + "x"+humrob[0][2] + "," + humrob[0][3]);
		// System.out.println(humrob[1][0] + "," +humrob[1][1] + "x"+humrob[1][2] + "," + humrob[1][3]);
		// System.out.println(humrob[2][0] + "," +humrob[2][1] + "x"+humrob[2][2] + "," + humrob[2][3]);
		// System.out.println(humrob[3][0] + "," +humrob[3][1] + "x"+humrob[3][2] + "," + humrob[3][3]);
		// System.out.println(humrob[4][0] + "," +humrob[4][1] + "x"+humrob[4][2] + "," + humrob[4][3]);
		
		double[] sabun = new double[5];		//ƗpBrobhumrob̋𔻒
		if(ij==0){
			System.out.println("Person1`5NȂ");
			return 0;							//ulԕs݁v̐MB͑̈ɉȂƂ
		}
		
		for(int i=0;i<ij;i++){
			//{bg̃{WƁAhumrob`FbNA
	    	sabun[i] = sqrt( ((rob[0] - humrob[i][0])*(rob[0] - humrob[i][0])) + ((rob[1] - humrob[i][1])*(rob[1] - humrob[i][1])) );
		}
		
		int thisisrobot=0;		//robothumanHȂ̂m
		for(int i=0;i<ij;i++){
			if(sabun[thisisrobot] > sabun[i]){
				thisisrobot =i;
			}
		}					//ŁAsabun[thisisrobot]ŏ
		if(sabun[thisisrobot]>1000){	//ԋ߂̂ł1000mm𒴂
			thisisrobot = 99;
		}else{						//łȂ΁Ahumrob[thisisrobot]robot
			System.out.println("person"+ (thisisrobot+1) +"{bgłB"+sabun[thisisrobot]);
		}
		
		for(int i=0;i<ij;i++){
			if(i == thisisrobot){		//̓{bgȂ̂humɈڂȂ
			}else if(i > thisisrobot){	//thisisrobotɂẮAhum
				hum[i-1][0] = humrob[i][0];
				hum[i-1][1] = humrob[i][1];
				hum[i-1][2] = humrob[i][2];
				hum[i-1][3] = humrob[i][3];
			}else{						//thisisrobotOȂAʂɓ΂悢
				hum[i][0] = humrob[i][0];
				hum[i][1] = humrob[i][1];
				hum[i][2] = humrob[i][2];
				hum[i][3] = humrob[i][3];
			}
		}
		if(thisisrobot<99){
			ij = ij -1;			//oij烍{bg
		}else{
			System.out.println("ZTϑ͈͓Ƀ{bg͂܂");
		}
		
		return ij;
	}
	
	private boolean tagquestion(String tagid ,double[] zahyou){
		diTagLocationInfoHolder loc = new diTagLocationInfoHolder();
		loc.value = new diTagLocationInfo();
		long res = m_if_diTag.getTagLocationByTag(tagid, loc);
		if (res == 0) {
			// System.out.println("ok.");
		} else {
			// System.out.println("error[" + res + "]");
			return false;
		}
		
		//Aoutۂ𕷂
		diContentObjectInfoHolder content = new diContentObjectInfoHolder();
		content.value = new diContentObjectInfo();
		long ren = m_if_diTag.getContentObjectByTag(tagid, content);
		if (ren == 0) {
			// System.out.println("dicontentObject ok.");
		} else {
			// System.out.println("error[" + ren + "]");
			return false;
		}
		diContentObjectInfo cabin = new diContentObjectInfo();
		cabin = content.value;
		if(!"out".equals(cabin.cabinetid)){		
			diTagLocationInfo info = new diTagLocationInfo();
			info = loc.value;
			zahyou[0] = info.coordinateX;
			zahyou[1] = info.coordinateY;
			zahyou[2] = info.velocityX;
			zahyou[3] = info.velocityY;
			return true;
		}else {
			return false;
		}
	}    
    
    
    
    
    
    /*	void SetLoginInfo()
	{
	
		long res = m_if_diManagement.setLoginInfo("robot1", "user1","abc");
		if (res == 0) {
			// System.out.println("ok.");
		} else {
			// System.out.println("error[" + res + "]");
		}
	}*/    
    
    /***
     *
     * The aborting action when main logic error occurred.
     * former rtc_aborting_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//  @Override
//  public ReturnCode_t onAborting(int ec_id) {
//      return super.onAborting(ec_id);
//  }

    /***
     *
     * The error action in ERROR state
     * former rtc_error_do()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    public ReturnCode_t onError(int ec_id) {
//        return super.onError(ec_id);
//    }

    /***
     *
     * The reset action that is invoked resetting
     * This is same but different the former rtc_init_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onReset(int ec_id) {
//        return super.onReset(ec_id);
//    }

    /***
     *
     * The state update action that is invoked after onExecute() action
     * no corresponding operation exists in OpenRTm-aist-0.2.0
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onStateUpdate(int ec_id) {
//        return super.onStateUpdate(ec_id);
//    }

    /***
     *
     * The action that is invoked when execution context's rate is changed
     * no corresponding operation exists in OpenRTm-aist-0.2.0
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onRateChanged(int ec_id) {
//        return super.onRateChanged(ec_id);
//    }
//
    // DataInPort declaration
    // <rtc-template block="inport_declare">
    
    // </rtc-template>

    // DataOutPort declaration
    // <rtc-template block="outport_declare">
    protected TimedLong m_upperlimit_val;
    protected DataRef<TimedLong> m_upperlimit;
    /*!
     */
    protected OutPort<TimedLong> m_upperlimitOut;

    protected TimedLong m_signal_val;
    protected DataRef<TimedLong> m_signal;
    /*!
     */
    protected OutPort<TimedLong> m_signalOut;

    
    // </rtc-template>

    // CORBA Port declaration
    // <rtc-template block="corbaport_declare">
    /*!
     * robotto no keirojyouhou ni kansuru syori wo okonau
     */
    protected CorbaPort m_spRobotPathPort;
    /*!
     */
    protected CorbaPort m_diManagementPort;
    /*!
     */
    protected CorbaPort m_diTagPort;
    /*!
     */
    protected CorbaPort m_diObjectPort;
    /*!
     */
    protected CorbaPort m_diRobotPort;
    
    // </rtc-template>

    // Service declaration
    // <rtc-template block="service_declare">
    /*!
     */
    protected spRobotPathSVC_impl m_spRobotPath = new spRobotPathSVC_impl();
    
    // </rtc-template>

    // Consumer declaration
    // <rtc-template block="consumer_declare">
    protected CorbaConsumer<diManagement> m_if_diManagementBase = new CorbaConsumer<diManagement>(diManagement.class);
    /*!
     */
    protected diManagement m_if_diManagement;
    protected CorbaConsumer<diTag> m_if_diTagBase = new CorbaConsumer<diTag>(diTag.class);
    /*!
     */
    protected diTag m_if_diTag;
    protected CorbaConsumer<diObject> m_if_diObjectBase = new CorbaConsumer<diObject>(diObject.class);
    /*!
     */
    protected diObject m_if_diObject;
    protected CorbaConsumer<diRobot> m_if_diRobotBase = new CorbaConsumer<diRobot>(diRobot.class);
    /*!
     */
    protected diRobot m_if_diRobot;
    
    // </rtc-template>


}
